/*
 * show_images_thr.c
 * The outer function of the image-showing thread.
 * bgrot project
 * $Fuller: myprograms/bgrot/server/show_images_thr.c,v 1.11 2003/03/10 13:18:02 fullermd Exp $
 */

#include <pthread.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include <bgrot.h>
#include <bgrotd.h>

/* Pull in globals we care about */
extern BGROTD_CONF *runcf;
extern pthread_mutex_t runcf_mtx;

extern IMGENT **cur_img;
extern IMGENT **nxt_img;
extern pthread_mutex_t cur_img_mtx;
extern pthread_cond_t cur_img_cond;

extern int debug;


/* Through the never */
void *
show_images_thr(void *dummy)
{
	IMGLIST *curlist;
	IMGENT *shownow, *shownext;
	INDEX_ENT *curlent=NULL;
	char *cffile;
	size_t mlen;
	int i, wlsize;
	int intvary, interval=0, roffset;
	char *display=NULL;
	pthread_attr_t ptattr;

	pthread_attr_init(&ptattr);
	pthread_attr_setdetachstate(&ptattr, PTHREAD_CREATE_DETACHED);

	/* Get everything we need from the global config */
	pthread_mutex_lock(&runcf_mtx);

	/* Figure out which list file we're actually using */
	for(i=0 ; !curlent ; i++)
	{
		if(runcf->sel_list_id == (runcf->list_index->lists)[i]->id)
			curlent = (runcf->list_index->lists)[i];
	}

	/* basedir + '/' + listdir + "/" + [listfile] + '\0' */
	mlen = strlen(runcf->basedir) + 1 + strlen(runcf->listdir) + 1
			+ strlen(curlent->filename) + 1; 
	cffile = malloc(mlen);
	sprintf(cffile, "%s/%s/%s", runcf->basedir, runcf->listdir,
			curlent->filename);

	/* Pull out config overrides and such */
	if(runcf->conf_override)
	{
		interval = runcf->conf_override->interval;
		intvary = runcf->conf_override->rvary;
		display = strdup(runcf->conf_override->display);
	}

	/* Done with config-reading */
	pthread_mutex_unlock(&runcf_mtx);

	/* Loop forever, really */
	while(1)
	{
		/* Parse out our image list file */
		curlist = parse_list(cffile);

		/* Check config */
		if(interval==0)
		{
			interval = curlist->conf->interval;
			intvary = curlist->conf->rvary;
		}
		if(display==NULL)
			display = strdup(curlist->conf->display);

		/* Set our $DISPLAY */
		setenv("DISPLAY", display, 1);

		/* Now, our inner loop; display all the images in this list */
		if(debug==1)
			fprintf(stderr, "Starting inner display loop...\n");
		wlsize = curlist->img_used;
		shownext = random_img(&curlist->imgs, &wlsize);
		for( i = (curlist->img_used-1) ; i>=0 ; i-- )
		{
			pthread_t disp_thr; /* $DEITY knows how many */

			shownow = shownext;
			if(i!=0)
				shownext = random_img(&curlist->imgs, &wlsize);

			/* I'm pretty sure we _do_ want to detach */
			/* See discussion in design/display-thread */
			if(debug==1)
				fprintf(stderr, "Spawning display of '%s'...\n",
						shownow->fname);
			pthread_create(&disp_thr, &ptattr,
					display_image_thr, (void *)shownow);

			/* There's gonna be a lag here.  So be it */
			pthread_mutex_lock(&cur_img_mtx);
			free_imgent(*cur_img);
			*cur_img = alloc_imgent();
			/* XXX We should write an imgent_dup() function */
			(*cur_img)->fname = strdup(shownow->fname);
			free_imgent(*nxt_img);
			*nxt_img = alloc_imgent();
			(*nxt_img)->fname = strdup(shownext->fname);
			pthread_mutex_unlock(&cur_img_mtx);

			/*
			 * shownow is free'd in the display_image_thr thread.
			 * shownext becomes next rotation's shownow.
			 */ 

			/* Figure out how long to sleep */
			roffset = intvary ? ((rand()%(intvary*2+1))-intvary) : intvary;
			if(debug==1)
				fprintf(stderr, "Sleeping for %d (%d + %d) seconds...\n",
						interval+roffset, interval, roffset);
			sleep(interval+roffset);
		}
	}

	/* XXX Never happens.  We need to make a cleanup function */
	free(cffile);

	/* Shouldn't ever really get here, I don't think... */
	exit(1);
}
